# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2015-2016. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import logging
import time
import argparse
import os
import numpy as np
import allensdk.core.json_utilities as json_utilities
from allensdk.core.nwb_data_set import NwbDataSet
from allensdk.api.queries.glif_api import GlifApi
from allensdk.model.glif.glif_neuron import GlifNeuron

DEFAULT_SPIKE_CUT_VALUE = 0.05 # 50mV

def parse_arguments():
    ''' Use argparse to get required arguments from the command line '''
    parser = argparse.ArgumentParser(description='fit a neuron')

    parser.add_argument('--ephys_file', help='ephys file name')
    parser.add_argument('--sweeps_file', help='JSON file listing sweep properties')
    parser.add_argument('--neuron_config_file', help='neuron configuration JSON file ')
    parser.add_argument('--neuronal_model_id', help='id of the neuronal model. Used when downloading sweep properties.', type=int)
    parser.add_argument('--output_ephys_file', help='output file name')
    parser.add_argument('--log_level', help='log level', default=logging.INFO)
    parser.add_argument('--spike_cut_value', help='value to fill in for spike duration', default=DEFAULT_SPIKE_CUT_VALUE, type=float)

    return parser.parse_args()


def simulate_sweep(neuron, stimulus, spike_cut_value):
    ''' Simulate a neuron given a stimulus and initial conditions. '''

    start_time = time.time()

    logging.debug("simulating")

    data = neuron.run(stimulus)

    voltage = data['voltage']
    voltage[np.isnan(voltage)] = spike_cut_value

    logging.debug("simulation time %f" % (time.time() - start_time))

    return data


def load_sweep(file_name, sweep_number):
    ''' Load the stimulus for a sweep from file. '''
    logging.debug("loading sweep %d" % sweep_number)

    load_start_time = time.time()
    data = NwbDataSet(file_name).get_sweep(sweep_number)

    logging.debug("load time %f" % (time.time() - load_start_time))

    return data


def write_sweep_response(file_name, sweep_number, response, spike_times):
    ''' Overwrite the response in a file. '''

    logging.debug("writing sweep")

    write_start_time = time.time()
    ephds = NwbDataSet(file_name)

    ephds.set_sweep(sweep_number, stimulus=None, response=response)
    ephds.set_spike_times(sweep_number, spike_times)

    logging.debug("write time %f" % (time.time() - write_start_time))


def simulate_sweep_from_file(neuron, sweep_number, input_file_name, output_file_name, spike_cut_value):
    ''' Load a sweep stimulus, simulate the response, and write it out. '''

    sweep_start_time = time.time()

    try:
        data = load_sweep(input_file_name, sweep_number)
    except Exception as e:
        logging.warning("Failed to load sweep, skipping. (%s)" % str(e))
        raise

        # tell the neuron what dt should be for this sweep
    neuron.dt = 1.0 / data['sampling_rate']

    sim_data = simulate_sweep(neuron, data['stimulus'], spike_cut_value)

    write_sweep_response(output_file_name, sweep_number, sim_data['voltage'], sim_data['interpolated_spike_times'])

    logging.debug("total sweep time %f" % ( time.time() - sweep_start_time ))

def simulate_neuron(neuron, sweep_numbers, input_file_name, output_file_name, spike_cut_value):

    start_time = time.time()

    for sweep_number in sweep_numbers:
        simulate_sweep_from_file(neuron, sweep_number, input_file_name, output_file_name, spike_cut_value)

    logging.debug("total elapsed time %f" % (time.time() - start_time))

def main():
    args = parse_arguments()

    logging.getLogger().setLevel(args.log_level)

    glif_api = None
    if (args.neuron_config_file is None or
        args.sweeps_file is None or
        args.ephys_file is None):

        assert args.neuronal_model_id is not None, Exception("A neuronal model id is required if no neuron config file, sweeps file, or ephys data file is provided.")

        glif_api = GlifApi()
        glif_api.get_neuronal_model(args.neuronal_model_id)

    if args.neuron_config_file:
        neuron_config = json_utilities.read(args.neuron_config_file)
    else:
        neuron_config = glif_api.get_neuron_config()

    if args.sweeps_file:
        sweeps = json_utilities.read(args.sweeps_file)
    else:
        sweeps = glif_api.get_ephys_sweeps()

    if args.ephys_file:
        ephys_file = args.ephys_file
    else:
        ephys_file = 'stimulus_%d.nwb' % args.neuronal_model_id

        if not os.path.exists(ephys_file):
            logging.info("Downloading stimulus to %s." % ephys_file)
            glif_api.cache_stimulus_file(ephys_file)
        else:
            logging.warning("Reusing %s because it already exists." % ephys_file)

    if args.output_ephys_file:
        output_ephys_file = args.output_ephys_file
    else:
        logging.warning("Overwriting input file data with simulated data in place.")
        output_ephys_file = ephys_file


    neuron = GlifNeuron.from_dict(neuron_config)

    # filter out test sweeps
    sweep_numbers = [ s['sweep_number'] for s in sweeps if s['stimulus_name'] != 'Test' ]

    simulate_neuron(neuron, sweep_numbers, ephys_file, output_ephys_file, args.spike_cut_value)



if __name__ == "__main__": main()



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2016-2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import scipy.stats as st
import numpy as np
import pandas as pd
import logging
from .findlevel import findlevel
from .brain_observatory_exceptions import BrainObservatoryAnalysisException
from . import observatory_plots as oplots
import matplotlib.pyplot as plt

class StimulusAnalysis(object):
    """ Base class for all response analysis code. Subclasses are responsible
    for computing metrics and traces relevant to a particular stimulus.
    The base class contains methods for organizing sweep responses row of
    a stimulus stable (get_sweep_response).  Subclasses implement the
    get_response method, computes the mean sweep response to all sweeps for
    a each stimulus condition.

    Parameters
    ----------
    data_set: BrainObservatoryNwbDataSet instance

    speed_tuning: boolean, deprecated
       Whether or not to compute speed tuning histograms

    """
    _log = logging.getLogger('allensdk.brain_observatory.stimulus_analysis')
    _PRELOAD = "PRELOAD"

    def __init__(self, data_set):
        self.data_set = data_set
        self._timestamps = StimulusAnalysis._PRELOAD
        self._celltraces = StimulusAnalysis._PRELOAD
        self._acquisition_rate = StimulusAnalysis._PRELOAD
        self._numbercells = StimulusAnalysis._PRELOAD
        self._roi_id = StimulusAnalysis._PRELOAD
        self._cell_id = StimulusAnalysis._PRELOAD
        self._dfftraces = StimulusAnalysis._PRELOAD
        self._dxcm = StimulusAnalysis._PRELOAD
        self._dxtime = StimulusAnalysis._PRELOAD
        self._binned_dx_sp = StimulusAnalysis._PRELOAD
        self._binned_cells_sp = StimulusAnalysis._PRELOAD
        self._binned_dx_vis = StimulusAnalysis._PRELOAD
        self._binned_cells_vis = StimulusAnalysis._PRELOAD
        self._peak_run = StimulusAnalysis._PRELOAD
        self._binsize = 800

        self._stim_table = StimulusAnalysis._PRELOAD
        self._response = StimulusAnalysis._PRELOAD
        self._sweep_response = StimulusAnalysis._PRELOAD
        self._mean_sweep_response = StimulusAnalysis._PRELOAD
        self._pval = StimulusAnalysis._PRELOAD
        self._peak = StimulusAnalysis._PRELOAD

    @property
    def stim_table(self):
        if self._stim_table is StimulusAnalysis._PRELOAD:
            self.populate_stimulus_table()

        return self._stim_table

    @property
    def sweep_response(self):
        if self._sweep_response is StimulusAnalysis._PRELOAD:
            self._sweep_response, self._mean_sweep_response, self._pval = \
                self.get_sweep_response()

        return self._sweep_response

    @property
    def mean_sweep_response(self):
        if self._mean_sweep_response is StimulusAnalysis._PRELOAD:
            self._sweep_response, self._mean_sweep_response, self._pval = \
                self.get_sweep_response()

        return self._mean_sweep_response

    @property
    def pval(self):
        if self._pval is StimulusAnalysis._PRELOAD:
            self._sweep_response, self._mean_sweep_response, self._pval = \
                self.get_sweep_response()

        return self._pval

    @property
    def response(self):
        if self._response is StimulusAnalysis._PRELOAD:
            self._response = self.get_response()

        return self._response

    @property
    def peak(self):
        if self._peak is StimulusAnalysis._PRELOAD:
            self._peak = self.get_peak()

        return self._peak

    def get_fluorescence(self):
        # get fluorescence
        self._timestamps, self._celltraces = \
            self.data_set.get_corrected_fluorescence_traces()
        self._acquisition_rate = 1 / (self.timestamps[1] - self.timestamps[0])
        self._numbercells = len(self.celltraces)  # number of cells in dataset

    @property
    def timestamps(self):
        if self._timestamps is StimulusAnalysis._PRELOAD:
            self.get_fluorescence()

        return self._timestamps

    @property
    def celltraces(self):
        if self._celltraces is StimulusAnalysis._PRELOAD:
            self.get_fluorescence()

        return self._celltraces

    @property
    def acquisition_rate(self):
        if self._acquisition_rate is StimulusAnalysis._PRELOAD:
            self.get_fluorescence()

        return self._acquisition_rate

    @property
    def numbercells(self):
        if self._numbercells is StimulusAnalysis._PRELOAD:
            self.get_fluorescence()

        return self._numbercells

    @property
    def roi_id(self):
        if self._roi_id is StimulusAnalysis._PRELOAD:
            self._roi_id = self.data_set.get_roi_ids()

        return self._roi_id

    @property
    def cell_id(self):
        if self._cell_id is StimulusAnalysis._PRELOAD:
            self._cell_id = self.data_set.get_cell_specimen_ids()

        return self._cell_id

    @property
    def dfftraces(self):
        if self._dfftraces is StimulusAnalysis._PRELOAD:
            _, self._dfftraces = self.data_set.get_dff_traces()

        return self._dfftraces

    @property
    def dxcm(self):
        if self._dxcm is StimulusAnalysis._PRELOAD:
            self._dxcm, self._dxtime = self.data_set.get_running_speed()

        return self._dxcm

    @property
    def dxtime(self):
        if self._dxtime is StimulusAnalysis._PRELOAD:
            self._dxcm, self._dxtime = self.data_set.get_running_speed()
            
        return self._dxtime

    @property
    def binned_dx_sp(self):
        if self._binned_dx_sp is StimulusAnalysis._PRELOAD:
            (self._binned_dx_sp, self._binned_cells_sp, self._binned_dx_vis,
             self._binned_cells_vis, self._peak_run) = \
                self.get_speed_tuning(binsize=self._binsize)

        return self._binned_dx_sp

    @property
    def binned_cells_sp(self):
        if self._binned_cells_sp is StimulusAnalysis._PRELOAD:
            (self._binned_dx_sp, self._binned_cells_sp, self._binned_dx_vis,
             self._binned_cells_vis, self._peak_run) = \
                self.get_speed_tuning(binsize=self._binsize)

        return self._binned_cells_sp

    @property
    def binned_dx_vis(self):
        if self._binned_dx_vis is StimulusAnalysis._PRELOAD:
            (self._binned_dx_sp, self._binned_cells_sp, self._binned_dx_vis,
             self._binned_cells_vis, self._peak_run) = \
                self.get_speed_tuning(binsize=self._binsize)

        return self._binned_dx_vis

    @property
    def binned_cells_vis(self):
        if self._binned_cells_vis is StimulusAnalysis._PRELOAD:
            (self._binned_dx_sp, self._binned_cells_sp, self._binned_dx_vis,
             self._binned_cells_vis, self._peak_run) = \
                self.get_speed_tuning(binsize=self._binsize)

        return self._binned_cells_vis

    @property
    def peak_run(self):
        if self._peak_run is StimulusAnalysis._PRELOAD:
            (self._binned_dx_sp, self._binned_cells_sp, self._binned_dx_vis,
             self._binned_cells_vis, self._peak_run) = \
                self.get_speed_tuning(binsize=self._binsize)

        return self._peak_run

    def populate_stimulus_table(self):
        """ Implemented by subclasses. """
        raise BrainObservatoryAnalysisException("populate_stimulus_table not implemented")

    def get_response(self):
        """ Implemented by subclasses. """
        raise BrainObservatoryAnalysisException("get_response not implemented")

    def get_peak(self):
        """ Implemented by subclasses. """
        raise BrainObservatoryAnalysisException("get_peak not implemented")

    def get_speed_tuning(self, binsize):
        """ Calculates speed tuning, spontaneous versus visually driven.  The return is a 5-tuple
        of speed and dF/F histograms.

            binned_dx_sp: (bins,2) np.ndarray of running speeds binned during spontaneous activity stimulus.
            The first bin contains all speeds below 1 cm/s.  Dimension 0 is mean running speed in the bin.
            Dimension 1 is the standard error of the mean.

            binned_cells_sp: (bins,2) np.ndarray of fluorescence during spontaneous activity stimulus.
            First bin contains all data for speeds below 1 cm/s. Dimension 0 is mean fluorescence in the bin.
            Dimension 1 is the standard error of the mean.

            binned_dx_vis: (bins,2) np.ndarray of running speeds outside of spontaneous activity stimulus.
            The first bin contains all speeds below 1 cm/s.  Dimension 0 is mean running speed in the bin.
            Dimension 1 is the standard error of the mean.

            binned_cells_vis: np.ndarray of fluorescence outside of spontaneous activity stimulu.
            First bin contains all data for speeds below 1 cm/s. Dimension 0 is mean fluorescence in the bin.
            Dimension 1 is the standard error of the mean.

            peak_run: pd.DataFrame of speed-related properties of a cell.

        Returns
        -------
        tuple: binned_dx_sp, binned_cells_sp, binned_dx_vis, binned_cells_vis, peak_run
        """

        StimulusAnalysis._log.info(
            'Calculating speed tuning, spontaneous vs visually driven')

        celltraces_trimmed = np.delete(self.dfftraces, range(
            len(self.dxcm), np.size(self.dfftraces, 1)), axis=1)

        # pull out spontaneous epoch(s)
        spontaneous = self.data_set.get_stimulus_table('spontaneous')

        peak_run = pd.DataFrame(index=range(self.numbercells), columns=(
            'speed_max_sp', 'speed_min_sp', 'ptest_sp', 'mod_sp', 'speed_max_vis', 'speed_min_vis', 'ptest_vis', 'mod_vis'))

        dx_sp = self.dxcm[spontaneous.start.iloc[-1]:spontaneous.end.iloc[-1]]
        celltraces_sp = celltraces_trimmed[
            :, spontaneous.start.iloc[-1]:spontaneous.end.iloc[-1]]
        dx_vis = np.delete(self.dxcm, np.arange(
            spontaneous.start.iloc[-1], spontaneous.end.iloc[-1]))
        celltraces_vis = np.delete(celltraces_trimmed, np.arange(
            spontaneous.start.iloc[-1], spontaneous.end.iloc[-1]), axis=1)
        if len(spontaneous) > 1:
            dx_sp = np.append(
                dx_sp, self.dxcm[spontaneous.start.iloc[-2]:spontaneous.end.iloc[-2]], axis=0)
            celltraces_sp = np.append(celltraces_sp, celltraces_trimmed[
                                      :, spontaneous.start.iloc[-2]:spontaneous.end.iloc[-2]], axis=1)
            dx_vis = np.delete(dx_vis, np.arange(
                spontaneous.start.iloc[-2], spontaneous.end.iloc[-2]))
            celltraces_vis = np.delete(celltraces_vis, np.arange(
                spontaneous.start.iloc[-2], spontaneous.end.iloc[-2]), axis=1)
        celltraces_vis = celltraces_vis[:, ~np.isnan(dx_vis)]
        dx_vis = dx_vis[~np.isnan(dx_vis)]

        nbins = 1 + len(np.where(dx_sp >= 1)[0]) / binsize
        dx_sorted = dx_sp[np.argsort(dx_sp)]
        celltraces_sorted_sp = celltraces_sp[:, np.argsort(dx_sp)]
        binned_cells_sp = np.zeros((self.numbercells, nbins, 2))
        binned_dx_sp = np.zeros((nbins, 2))
        for i in range(nbins):
            if np.all(np.isnan(dx_sorted)):
                raise BrainObservatoryAnalysisException(
                    "dx is filled with NaNs")

            offset = findlevel(dx_sorted, 1, 'up')

            if offset is None:
                StimulusAnalysis._log.info(
                    "dx never crosses 1, all speed data going into single bin")
                offset = len(dx_sorted)

            if i == 0:
                binned_dx_sp[i, 0] = np.mean(dx_sorted[:offset])
                binned_dx_sp[i, 1] = np.std(
                    dx_sorted[:offset]) / np.sqrt(offset)
                binned_cells_sp[:, i, 0] = np.mean(
                    celltraces_sorted_sp[:, :offset], axis=1)
                binned_cells_sp[:, i, 1] = np.std(
                    celltraces_sorted_sp[:, :offset], axis=1) / np.sqrt(offset)
            else:
                start = offset + (i - 1) * binsize
                binned_dx_sp[i, 0] = np.mean(dx_sorted[start:start + binsize])
                binned_dx_sp[i, 1] = np.std(
                    dx_sorted[start:start + binsize]) / np.sqrt(binsize)
                binned_cells_sp[:, i, 0] = np.mean(
                    celltraces_sorted_sp[:, start:start + binsize], axis=1)
                binned_cells_sp[:, i, 1] = np.std(
                    celltraces_sorted_sp[:, start:start + binsize], axis=1) / np.sqrt(binsize)

        binned_cells_shuffled_sp = np.empty((self.numbercells, nbins, 2, 200))
        for shuf in range(200):
            celltraces_shuffled = celltraces_sp[
                :, np.random.permutation(np.size(celltraces_sp, 1))]
            celltraces_shuffled_sorted = celltraces_shuffled[
                :, np.argsort(dx_sp)]
            for i in range(nbins):
                offset = findlevel(dx_sorted, 1, 'up')

                if offset is None:
                    StimulusAnalysis._log.info(
                        "dx never crosses 1, all speed data going into single bin")
                    offset = celltraces_shuffled_sorted.shape[1]

                if i == 0:
                    binned_cells_shuffled_sp[:, i, 0, shuf] = np.mean(
                        celltraces_shuffled_sorted[:, :offset], axis=1)
                    binned_cells_shuffled_sp[:, i, 1, shuf] = np.std(
                        celltraces_shuffled_sorted[:, :offset], axis=1)
                else:
                    start = offset + (i - 1) * binsize
                    binned_cells_shuffled_sp[:, i, 0, shuf] = np.mean(
                        celltraces_shuffled_sorted[:, start:start + binsize], axis=1)
                    binned_cells_shuffled_sp[:, i, 1, shuf] = np.std(
                        celltraces_shuffled_sorted[:, start:start + binsize], axis=1)

        nbins = 1 + len(np.where(dx_vis >= 1)[0]) / binsize
        dx_sorted = dx_vis[np.argsort(dx_vis)]
        celltraces_sorted_vis = celltraces_vis[:, np.argsort(dx_vis)]
        binned_cells_vis = np.zeros((self.numbercells, nbins, 2))
        binned_dx_vis = np.zeros((nbins, 2))
        for i in range(nbins):
            offset = findlevel(dx_sorted, 1, 'up')

            if offset is None:
                StimulusAnalysis._log.info(
                    "dx never crosses 1, all speed data going into single bin")
                offset = len(dx_sorted)

            if i == 0:
                binned_dx_vis[i, 0] = np.mean(dx_sorted[:offset])
                binned_dx_vis[i, 1] = np.std(
                    dx_sorted[:offset]) / np.sqrt(offset)
                binned_cells_vis[:, i, 0] = np.mean(
                    celltraces_sorted_vis[:, :offset], axis=1)
                binned_cells_vis[:, i, 1] = np.std(
                    celltraces_sorted_vis[:, :offset], axis=1) / np.sqrt(offset)
            else:
                start = offset + (i - 1) * binsize
                binned_dx_vis[i, 0] = np.mean(dx_sorted[start:start + binsize])
                binned_dx_vis[i, 1] = np.std(
                    dx_sorted[start:start + binsize]) / np.sqrt(binsize)
                binned_cells_vis[:, i, 0] = np.mean(
                    celltraces_sorted_vis[:, start:start + binsize], axis=1)
                binned_cells_vis[:, i, 1] = np.std(
                    celltraces_sorted_vis[:, start:start + binsize], axis=1) / np.sqrt(binsize)

        binned_cells_shuffled_vis = np.empty((self.numbercells, nbins, 2, 200))
        for shuf in range(200):
            celltraces_shuffled = celltraces_vis[
                :, np.random.permutation(np.size(celltraces_vis, 1))]
            celltraces_shuffled_sorted = celltraces_shuffled[
                :, np.argsort(dx_vis)]
            for i in range(nbins):
                offset = findlevel(dx_sorted, 1, 'up')

                if offset is None:
                    StimulusAnalysis._log.info(
                        "dx never crosses 1, all speed data going into single bin")
                    offset = len(dx_sorted)

                if i == 0:
                    binned_cells_shuffled_vis[:, i, 0, shuf] = np.mean(
                        celltraces_shuffled_sorted[:, :offset], axis=1)
                    binned_cells_shuffled_vis[:, i, 1, shuf] = np.std(
                        celltraces_shuffled_sorted[:, :offset], axis=1)
                else:
                    start = offset + (i - 1) * binsize
                    binned_cells_shuffled_vis[:, i, 0, shuf] = np.mean(
                        celltraces_shuffled_sorted[:, start:start + binsize], axis=1)
                    binned_cells_shuffled_vis[:, i, 1, shuf] = np.std(
                        celltraces_shuffled_sorted[:, start:start + binsize], axis=1)

        shuffled_variance_sp = binned_cells_shuffled_sp[
            :, :, 0, :].std(axis=1)**2
        variance_threshold_sp = np.percentile(
            shuffled_variance_sp, 99.9, axis=1)
        response_variance_sp = binned_cells_sp[:, :, 0].std(axis=1)**2

        shuffled_variance_vis = binned_cells_shuffled_vis[
            :, :, 0, :].std(axis=1)**2
        variance_threshold_vis = np.percentile(
            shuffled_variance_vis, 99.9, axis=1)
        response_variance_vis = binned_cells_vis[:, :, 0].std(axis=1)**2

        for nc in range(self.numbercells):
            if response_variance_vis[nc] > variance_threshold_vis[nc]:
                peak_run.mod_vis[nc] = True
            if response_variance_vis[nc] <= variance_threshold_vis[nc]:
                peak_run.mod_vis[nc] = False
            if response_variance_sp[nc] > variance_threshold_sp[nc]:
                peak_run.mod_sp[nc] = True
            if response_variance_sp[nc] <= variance_threshold_sp[nc]:
                peak_run.mod_sp[nc] = False
            temp = binned_cells_sp[nc, :, 0]
            start_max = temp.argmax()
            peak_run.speed_max_sp[nc] = binned_dx_sp[start_max, 0]
            start_min = temp.argmin()
            peak_run.speed_min_sp[nc] = binned_dx_sp[start_min, 0]
            if peak_run.speed_max_sp[nc] > peak_run.speed_min_sp[nc]:
                test_values = celltraces_sorted_sp[
                    nc, start_max * binsize:(start_max + 1) * binsize]
                other_values = np.delete(celltraces_sorted_sp[nc, :], range(
                    start_max * binsize, (start_max + 1) * binsize))
                (_, peak_run.ptest_sp[nc]) = st.ks_2samp(
                    test_values, other_values)
            else:
                test_values = celltraces_sorted_sp[
                    nc, start_min * binsize:(start_min + 1) * binsize]
                other_values = np.delete(celltraces_sorted_sp[nc, :], range(
                    start_min * binsize, (start_min + 1) * binsize))
                (_, peak_run.ptest_sp[nc]) = st.ks_2samp(
                    test_values, other_values)
            temp = binned_cells_vis[nc, :, 0]
            start_max = temp.argmax()
            peak_run.speed_max_vis[nc] = binned_dx_vis[start_max, 0]
            start_min = temp.argmin()
            peak_run.speed_min_vis[nc] = binned_dx_vis[start_min, 0]
            if peak_run.speed_max_vis[nc] > peak_run.speed_min_vis[nc]:
                test_values = celltraces_sorted_vis[
                    nc, start_max * binsize:(start_max + 1) * binsize]
                other_values = np.delete(celltraces_sorted_vis[nc, :], range(
                    start_max * binsize, (start_max + 1) * binsize))
            else:
                test_values = celltraces_sorted_vis[
                    nc, start_min * binsize:(start_min + 1) * binsize]
                other_values = np.delete(celltraces_sorted_vis[nc, :], range(
                    start_min * binsize, (start_min + 1) * binsize))
            (_, peak_run.ptest_vis[nc]) = st.ks_2samp(
                test_values, other_values)

        return binned_dx_sp, binned_cells_sp, binned_dx_vis, binned_cells_vis, peak_run

    def get_sweep_response(self):
        """ Calculates the response to each sweep in the stimulus table for each cell and the mean response.
        The return is a 3-tuple of:

            * sweep_response: pd.DataFrame of response dF/F traces organized by cell (column) and sweep (row)

            * mean_sweep_response: mean values of the traces returned in sweep_response

            * pval: p value from 1-way ANOVA comparing response during sweep to response prior to sweep

        Returns
        -------
        3-tuple: sweep_response, mean_sweep_response, pval
        """
        def do_mean(x):
            # +1])
            return np.mean(x[self.interlength:self.interlength + self.sweeplength + self.extralength])

        def do_p_value(x):
            (_, p) = st.f_oneway(x[:self.interlength], x[
                self.interlength:self.interlength + self.sweeplength + self.extralength])
            return p

        StimulusAnalysis._log.info('Calculating responses for each sweep')
        sweep_response = pd.DataFrame(index=self.stim_table.index.values, 
                                      columns=map(str, range(self.numbercells + 1)))
        sweep_response.rename(
            columns={str(self.numbercells): 'dx'}, inplace=True)
        for index, row in self.stim_table.iterrows():
            start = int(row['start'] - self.interlength)
            end = int(row['start'] + self.sweeplength + self.interlength)

            for nc in range(self.numbercells):
                temp = self.celltraces[int(nc), start:end]
                sweep_response[str(nc)][index] = 100 * \
                    ((temp / np.mean(temp[:self.interlength])) - 1)
            sweep_response['dx'][index] = self.dxcm[start:end]

        mean_sweep_response = sweep_response.applymap(do_mean)

        pval = sweep_response.applymap(do_p_value)
        return sweep_response, mean_sweep_response, pval

    def plot_representational_similarity(self, repsim, stimulus=False):
        if stimulus:
            pass

        ax = plt.gca()
        ax.imshow(repsim, interpolation='nearest', cmap='plasma')

    def plot_running_speed_histogram(self, xlim=None, nbins=None):
        if xlim is None:
            xlim = [-10,100]
        if nbins is None:
            nbins = 40

        ax = plt.gca()
        ax.hist(self.dxcm, bins=nbins, range=xlim, color=oplots.STIM_COLOR)
        ax.set_xlim(xlim)
        plt.xlabel("running speed (cm/s)")
        plt.ylabel("time points")

    def plot_speed_tuning(self, cell_specimen_id=None, 
                          cell_index=None,
                          evoked_color=oplots.EVOKED_COLOR, 
                          spontaneous_color=oplots.SPONTANEOUS_COLOR):
        cell_index = self.row_from_cell_id(cell_specimen_id, cell_index)

        oplots.plot_combined_speed(self.binned_cells_vis[cell_index,:,:]*100, self.binned_dx_vis[:,:], 
                                   self.binned_cells_sp[cell_index,:,:]*100, self.binned_dx_sp[:,:],
                                   evoked_color, spontaneous_color)

        ax = plt.gca()
        plt.xlabel("running speed (cm/s)")
        plt.ylabel("percent dF/F")

    def row_from_cell_id(self, csid=None, idx=None):

        if csid is not None and not np.isnan(csid):
            return self.data_set.get_cell_specimen_ids().tolist().index(csid)
        elif idx is not None:
            return idx
        else:
            raise Exception("Could not find row for csid(%s) idx(%s)" % (str(csid), str(idx)))
    



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import logging
from json import dump, dumps
from allensdk.config.model.description_parser import DescriptionParser
from allensdk.config.model.description import Description
from allensdk.core.json_utilities import JsonComments


class JsonDescriptionParser(DescriptionParser):
    log = logging.getLogger(__name__)

    def __init__(self):
        super(JsonDescriptionParser, self).__init__()

    def read(self, file_path, description=None, section=None, **kwargs):
        '''Parse a complete or partial configuration.

        Parameters
        ----------
        json_string : string
            Input to parse.
        description : Description, optional
            Where to put the parsed configuration.  If None a new one is created.
        section : string, optional
            Where to put the parsed configuration within the description.

        Returns
        -------
        Description
            The input description with parsed configuration added.

        Section is only specified for "bare" objects that are to be added to a section array.
        '''
        if description is None:
            description = Description()

        data = JsonComments.read_file(file_path)
        description.unpack(data, section)

        return description

    def read_string(self, json_string, description=None, section=None, **kwargs):
        '''Parse a complete or partial configuration.

        Parameters
        ----------
        json_string : string
            Input to parse.
        description : Description, optional
            Where to put the parsed configuration.  If None a new one is created.
        section : string, optional
            Where to put the parsed configuration within the description.

        Returns
        -------
        Description
            The input description with parsed configuration added.

        Section is only specified for "bare" objects that are to be added to a section array.
        '''
        if description is None:
            description = Description()

        data = JsonComments.read_string(json_string)

        description.unpack(data, section)

        return description

    def write(self, filename, description):
        '''Write the description to a JSON file.

        Parameters
        ----------
        description : Description
            Object to write.
        '''
        try:
            with open(filename, 'w') as f:
                dump(description.data, f, indent=2)

        except Exception:
            self.log.warn(
                "Couldn't write allensdk json description: %s" % filename)
            raise

        return

    def write_string(self, description):
        '''Write the description to a JSON string.

        Parameters
        ----------
        description : Description
            Object to write.

        Returns
        -------
        string
           JSON serialization of the input.
        '''
        try:
            json_string = dumps(description.data,
                                indent=2)
            return json_string
        except Exception:
            self.log.warn("Couldn't write allensdk json description: %s")
            raise



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2016-2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import logging
import allensdk.brain_observatory.stimulus_info as stimulus_info
import h5py
import numpy as np
import pandas as pd
import scipy.ndimage
from .receptive_field_analysis.receptive_field import compute_receptive_field_with_postprocessing
from .receptive_field_analysis.visualization import plot_receptive_field_data

from . import circle_plots as cplots
from . import observatory_plots as oplots
from .brain_observatory_exceptions import MissingStimulusException
from .stimulus_analysis import StimulusAnalysis
from .receptive_field_analysis.tools import dict_generator, read_h5_group
from scipy.stats.mstats import zscore

import matplotlib.pyplot as plt

class LocallySparseNoise(StimulusAnalysis):
    """ Perform tuning analysis specific to the locally sparse noise stimulus.

    Parameters
    ----------
    data_set: BrainObservatoryNwbDataSet object

    stimulus: string
       Name of locally sparse noise stimulus.  See brain_observatory.stimulus_info.

    nrows: int
       Number of rows in the stimulus template

    ncol: int
       Number of columns in the stimulus template
    """

    LSN_ON = 255
    LSN_OFF = 0
    LSN_GREY = 127
    LSN_OFF_SCREEN = 64

    def __init__(self, data_set, stimulus=None, **kwargs):
        super(LocallySparseNoise, self).__init__(data_set, **kwargs)
        if stimulus is None:
            self.stimulus = stimulus_info.LOCALLY_SPARSE_NOISE
        else:
            self.stimulus = stimulus

        try:
            lsn_dims = stimulus_info.LOCALLY_SPARSE_NOISE_DIMENSIONS[self.stimulus]
        except KeyError as e:
            raise KeyError("Unknown stimulus name: %s" % self.stimulus)
        
        self.nrows = lsn_dims[0]
        self.ncols = lsn_dims[1]
        
        self._LSN = LocallySparseNoise._PRELOAD
        self._LSN_mask = LocallySparseNoise._PRELOAD
        self._sweeplength = LocallySparseNoise._PRELOAD
        self._interlength = LocallySparseNoise._PRELOAD
        self._extralength = LocallySparseNoise._PRELOAD
        self._mean_response = LocallySparseNoise._PRELOAD
        self._receptive_field = LocallySparseNoise._PRELOAD
        self._cell_index_receptive_field_analysis_data = LocallySparseNoise._PRELOAD

    @property
    def LSN(self):
        if self._LSN is LocallySparseNoise._PRELOAD:
            self.populate_stimulus_table()

        return self._LSN

    @property
    def LSN_mask(self):
        if self._LSN_mask is LocallySparseNoise._PRELOAD:
            self.populate_stimulus_table()

        return self._LSN_mask

    @property
    def sweeplength(self):
        if self._sweeplength is LocallySparseNoise._PRELOAD:
            self.populate_stimulus_table()

        return self._sweeplength

    @property
    def interlength(self):
        if self._interlength is LocallySparseNoise._PRELOAD:
            self.populate_stimulus_table()

        return self._interlength

    @property
    def extralength(self):
        if self._extralength is LocallySparseNoise._PRELOAD:
            self.populate_stimulus_table()

        return self._extralength

    @property
    def receptive_field(self):
        if self._receptive_field is LocallySparseNoise._PRELOAD:
            self._receptive_field = self.get_receptive_field()

        return self._receptive_field

    @property
    def cell_index_receptive_field_analysis_data(self):
        if self._cell_index_receptive_field_analysis_data is LocallySparseNoise._PRELOAD:
            self._cell_index_receptive_field_analysis_data = self.get_receptive_field_analysis_data()

        return self._cell_index_receptive_field_analysis_data

    @property
    def mean_response(self):
        if self._mean_response is LocallySparseNoise._PRELOAD:
            self._mean_response = self.get_mean_response()

        return self._mean_response


    def get_peak(self):
        LocallySparseNoise._log.info('Calculating peak response properties')

        peak = pd.DataFrame(index=range(self.numbercells), columns=('rf_center_on_x_lsn', 'rf_center_on_y_lsn',
                                                                    'rf_center_off_x_lsn', 'rf_center_off_y_lsn',
                                                                    'rf_area_on_lsn', 'rf_area_off_lsn',
                                                                    'rf_distance_lsn', 'rf_overlap_index_lsn',
                                                                    'rf_chi2_lsn',
                                                                    'cell_specimen_id'))
        csids = self.data_set.get_cell_specimen_ids()

        df = self.get_receptive_field_attribute_df()
        peak.cell_specimen_id = csids

        for nc in range(self.numbercells):
            peak['rf_chi2_lsn'].iloc[nc] = df['chi_squared_analysis/min_p'].iloc[nc]

            # find the index of the largest on subunit, if it exists
            if 'on/gaussian_fit/area' in df.columns:
                area_on = df['on/gaussian_fit/area'].iloc[nc]
                # watch out for NaNs
                on_i = np.argmax(area_on) if isinstance(area_on, np.ndarray) else None
            else:
                on_i = None

            if on_i is None:
                peak['rf_area_on_lsn'].iloc[nc] = np.nan
                peak['rf_center_on_x_lsn'].iloc[nc] = np.nan
                peak['rf_center_on_y_lsn'].iloc[nc] = np.nan
            else:
                peak['rf_area_on_lsn'].iloc[nc] = df['on/gaussian_fit/area'].iloc[nc][on_i]
                peak['rf_center_on_x_lsn'].iloc[nc] = df['on/gaussian_fit/center_x'].iloc[nc][on_i]
                peak['rf_center_on_y_lsn'].iloc[nc] = df['on/gaussian_fit/center_y'].iloc[nc][on_i]

            # find the index of the largest off subunit, if it exists
            if 'off/gaussian_fit/area' in df.columns:
                area_off = df['off/gaussian_fit/area'].iloc[nc]
                # watch out for NaNs
                off_i = np.argmax(area_off) if isinstance(area_off, np.ndarray) else None
            else:
                off_i = None

            if off_i is None:
                peak['rf_area_off_lsn'].iloc[nc] = np.nan
                peak['rf_center_off_x_lsn'].iloc[nc] = np.nan
                peak['rf_center_off_y_lsn'].iloc[nc] = np.nan
            else:
                peak['rf_area_off_lsn'].iloc[nc] = df['off/gaussian_fit/area'].iloc[nc][off_i]
                peak['rf_center_off_x_lsn'].iloc[nc] = df['off/gaussian_fit/center_x'].iloc[nc][off_i]
                peak['rf_center_off_y_lsn'].iloc[nc] = df['off/gaussian_fit/center_y'].iloc[nc][off_i]


            if on_i is not None and off_i is not None:
                peak['rf_distance_lsn'].iloc[nc] = df['on/gaussian_fit/distance'].iloc[nc][on_i][off_i]
                peak['rf_overlap_index_lsn'].iloc[nc] = df['on/gaussian_fit/overlap'].iloc[nc][on_i][off_i]
            else:
                peak['rf_distance_lsn'].iloc[nc] = np.nan
                peak['rf_overlap_index_lsn'].iloc[nc] = np.nan

        return peak

    def populate_stimulus_table(self):
        self._stim_table = self.data_set.get_stimulus_table(self.stimulus)
        self._LSN, self._LSN_mask = self.data_set.get_locally_sparse_noise_stimulus_template(
            self.stimulus, mask_off_screen=False)
        self._sweeplength = self._stim_table['end'][
            1] - self._stim_table['start'][1]
        self._interlength = 4 * self._sweeplength
        self._extralength = self._sweeplength


    def get_mean_response(self):
        logging.debug("Calculating mean responses")
        mean_response = np.empty(
            (self.nrows, self.ncols, self.numbercells + 1, 2))

        for xp in range(self.nrows):
            for yp in range(self.ncols):
                logging.debug("  for pixel (%d,%d)" % (xp, yp))
                on_frame = np.where(self.LSN[:, xp, yp] == self.LSN_ON)[0]
                off_frame = np.where(self.LSN[:, xp, yp] == self.LSN_OFF)[0]
                subset_on = self.mean_sweep_response[
                    self.stim_table.frame.isin(on_frame)]
                subset_off = self.mean_sweep_response[
                    self.stim_table.frame.isin(off_frame)]
                mean_response[xp, yp, :, 0] = subset_on.mean(axis=0)
                mean_response[xp, yp, :, 1] = subset_off.mean(axis=0)
        return mean_response

    def get_receptive_field(self):
        ''' Calculates receptive fields for each cell
        '''

        receptive_field = np.zeros((self.nrows, self.ncols, self.numbercells, 2))

        for cell_index in range(len(self.cell_index_receptive_field_analysis_data)):
            curr_rf = self.cell_index_receptive_field_analysis_data[str(cell_index)]
            rf_on = curr_rf['on']['rts_convolution']['data'].copy()
            rf_off = curr_rf['off']['rts_convolution']['data'].copy()
            rf_on[np.logical_not(curr_rf['on']['fdr_mask']['data'].sum(axis=0))] = np.nan
            rf_off[np.logical_not(curr_rf['off']['fdr_mask']['data'].sum(axis=0))] = np.nan
            receptive_field[:,:,cell_index, 0] = rf_on
            receptive_field[:, :, cell_index, 1] = rf_off

        return receptive_field


    def get_receptive_field_analysis_data(self):
        ''' Calculates receptive fields for each cell
        '''

        csid_rf = {}
        for cell_index in range(self.data_set.number_of_cells):
            csid_rf[str(cell_index)] = compute_receptive_field_with_postprocessing(
                self.data_set, cell_index, self.stimulus, alpha=.05, number_of_shuffles=10000)

        return csid_rf


    def plot_receptive_field_analysis_data(self, cell_index, **kwargs):
        rf = self._cell_index_receptive_field_analysis_data[str(cell_index)]
        return plot_receptive_field_data(rf, self, **kwargs)

    def get_receptive_field_attribute_df(self):

        df_list = []
        for cell_index_as_str, rf in self.cell_index_receptive_field_analysis_data.items():

            attribute_dict = {}
            for x in dict_generator(rf):
                if x[-3] == 'attrs':
                    if len(x[:-3]) == 0:
                        key = x[-2]
                    else:
                        key = '/'.join(['/'.join(x[:-3]), x[-2]])
                    attribute_dict[key] = x[-1]

            massaged_dict = {}
            for key, val in attribute_dict.items():
                massaged_dict[key] = [val]

            massaged_dict['oeid'] = self.data_set.get_metadata()['ophys_experiment_id']

            curr_df = pd.DataFrame.from_dict(massaged_dict)
            df_list.append(curr_df)

            attribute_df = pd.concat(df_list)


        return attribute_df.sort(columns=['cell_index'])

    @staticmethod
    def merge_mean_response(rc1, rc2):
        """ Move out of this class, to session analysis
        """

        # make sure that rc1 is the larger one
        if rc2.shape[0] > rc1.shape[0]:
            rc1, rc2 = rc2, rc1

        shape_mult = np.array(rc1.shape) / np.array(rc2.shape)

        rc2_zoom = scipy.ndimage.zoom(rc2, shape_mult, order=0)

        return rc1 + rc2_zoom

    def plot_cell_receptive_field(self, on, cell_specimen_id=None, color_map=None, clim=None, mask=None, cell_index=None, scalebar=True):
        if color_map is None:
            color_map = 'Reds' if on else 'Blues'

        onst = 'on' if on else 'off'
        cell_idx = self.row_from_cell_id(cell_specimen_id, cell_index)
        rf = self.cell_index_receptive_field_analysis_data[str(cell_idx)]
        rts = rf[onst]['rts']['data']
        rts[np.logical_not(rf[onst]['fdr_mask']['data'].sum(axis=0))] = np.nan

        oplots.plot_receptive_field(rts, 
                                    color_map=color_map, 
                                    clim=clim, 
                                    mask=mask,
                                    scalebar=scalebar)

    def plot_population_receptive_field(self, color_map='RdPu', clim=None, mask=None, scalebar=True):
        rf = np.nansum(self.receptive_field, axis=(2,3))
        oplots.plot_receptive_field(rf,
                                    color_map=color_map,
                                    clim=clim,
                                    mask=mask,
                                    scalebar=scalebar)

    def sort_trials(self):
        ds = self.data_set

        lsn_movie, lsn_mask = ds.get_locally_sparse_noise_stimulus_template(self.stimulus, 
                                                                            mask_off_screen=False)

        baseline_trials = np.unique(np.where(lsn_movie[:,-5:,-1] != LocallySparseNoise.LSN_GREY)[0])
        baseline_df = self.mean_sweep_response.ix[baseline_trials]
        cell_baselines = np.nanmean(baseline_df.values, axis=0)

        lsn_movie[:,~lsn_mask] = LocallySparseNoise.LSN_OFF_SCREEN

        trials = {}
        for row in range(self.nrows):
            for col in range(self.ncols):
                on_trials = np.where(lsn_movie[:,row,col] == LocallySparseNoise.LSN_ON)
                off_trials = np.where(lsn_movie[:,row,col] == LocallySparseNoise.LSN_OFF)

                trials[(col,row,True)] = on_trials
                trials[(col,row,False)] = off_trials

        return trials, cell_baselines


    def open_pincushion_plot(self, on, cell_specimen_id=None, color_map=None, cell_index=None):
        cell_index = self.row_from_cell_id(cell_specimen_id, cell_index)

        trials, baselines = self.sort_trials()
        data = self.mean_sweep_response[str(cell_index)].values
        
        cplots.make_pincushion_plot(data, trials, on, 
                                    self.nrows, self.ncols,
                                    clim=[ baselines[cell_index], data.mean() + data.std() * 3 ],
                                    color_map=color_map,
                                    radius=1.0/16.0)

    @staticmethod
    def from_analysis_file(data_set, analysis_file, stimulus):
        lsn = LocallySparseNoise(data_set, stimulus)

        lsn.populate_stimulus_table()

        if stimulus == stimulus_info.LOCALLY_SPARSE_NOISE:
            stimulus_suffix = stimulus_info.LOCALLY_SPARSE_NOISE_SHORT
        elif stimulus == stimulus_info.LOCALLY_SPARSE_NOISE_4DEG:
            stimulus_suffix = stimulus_info.LOCALLY_SPARSE_NOISE_4DEG_SHORT
        elif stimulus == stimulus_info.LOCALLY_SPARSE_NOISE_8DEG:
            stimulus_suffix = stimulus_info.LOCALLY_SPARSE_NOISE_8DEG_SHORT

        try:

            with h5py.File(analysis_file, "r") as f:
                k = "analysis/mean_response_%s" % stimulus_suffix
                if k in f:
                    lsn._mean_response = f[k].value

            lsn._sweep_response = pd.read_hdf(analysis_file, "analysis/sweep_response_%s" % stimulus_suffix)
            lsn._mean_sweep_response = pd.read_hdf(analysis_file, "analysis/mean_sweep_response_%s" % stimulus_suffix)

            with h5py.File(analysis_file, "r") as f:
                lsn._cell_index_receptive_field_analysis_data = LocallySparseNoise.read_cell_index_receptive_field_analysis(f, stimulus)

        except Exception as e:
            raise MissingStimulusException(e.args)

        return lsn

    @staticmethod
    def save_cell_index_receptive_field_analysis(cell_index_receptive_field_analysis_data, new_nwb, prefix):

        attr_list = []
        file_handle = h5py.File(new_nwb.nwb_file, 'a')
        if prefix in file_handle['analysis']:
            del file_handle['analysis'][prefix]
        f = file_handle.create_group('analysis/%s' % prefix)
        for x in dict_generator(cell_index_receptive_field_analysis_data):
            if x[-2] == 'data':
                f['/'.join(x[:-1])] = x[-1]
            elif x[-3] == 'attrs':
                attr_list.append(x)
            else:
                raise Exception

        for x in attr_list:
            if len(x) > 3:
                f['/'.join(x[:-3])].attrs[x[-2]] = x[-1]
            else:
                assert len(x) == 3

                if x[-1] is None:
                    f.attrs[x[-2]] = np.NaN
                else:
                    f.attrs[x[-2]] = x[-1]

        file_handle.close()
        

    @staticmethod
    def read_cell_index_receptive_field_analysis(file_handle, prefix, path=None):
        k = 'analysis/%s' % prefix
        if k in file_handle:
            f = file_handle['analysis/%s' % prefix]
            if path is None:
                rf = read_h5_group(f)
            else:
                rf = read_h5_group(f[path])

            return rf
        else:
            return None

            




# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
from .rma_api import RmaApi
from ..cache import cacheable


class AnnotatedSectionDataSetsApi(RmaApi):
    '''See:
    `Searching Annotated SectionDataSets <http://help.brain-map.org/display/api/Searching+Annotated+SectionDataSets>`_
    '''

    def __init__(self, base_uri=None):
        super(AnnotatedSectionDataSetsApi, self).__init__(base_uri)

    def get_annotated_section_data_sets(self,
                                        structures,
                                        intensity_values=None,
                                        density_values=None,
                                        pattern_values=None,
                                        age_names=None):
        '''For a list of target structures, find the SectionDataSet
        that matches the parameters for intensity_values, density_values, pattern_values, and Age.

        Parameters
        ----------
        structure_graph_id : dict of integers
            what to retrieve
        intensity_values : array of strings, optional
            'High','Low', 'Medium' (default)
        density_values : array of strings, optional
            'High', 'Low'
        pattern_values : array of strings, optional
            'Full'
        age_names : array of strings, options
            for example 'E11.5', '13.5'

        Returns
        -------
        data : dict
            The parsed JSON repsonse message.

        Notes
        -----
        This method uses the non-RMA Annotated SectionDataSet endpoint.
        '''
        params = ['structures=' + ','.join((str(s) for s in structures))]

        if intensity_values is not None and len(intensity_values) > 0:
            params.append('intensity_values=' +
                          ','.join(("'%s'" % (v) for v in intensity_values)))

        if density_values is not None and len(density_values) > 0:
            params.append('density_values=' +
                          ','.join(("'%s'" % (v) for v in density_values)))

        if pattern_values is not None and len(pattern_values) > 0:
            params.append('pattern_values=' +
                          ','.join(("'%s'" % (v) for v in pattern_values)))

        if age_names is not None and len(age_names) > 0:
            params.append('age_names=' +
                          ','.join(("'%s'" % (v) for v in age_names)))

        url_params = '?' + '&'.join(params)

        url = ''.join([self.annotated_section_data_sets_endpoint,
                       '.json',
                       url_params])

        return self.json_msg_query(url)

    @cacheable()
    def get_annotated_section_data_sets_via_rma(self,
                                                structures,
                                                intensity_values=None,
                                                density_values=None,
                                                pattern_values=None,
                                                age_names=None):
        '''For a list of target structures, find the SectionDataSet
        that matches the parameters for intensity_values, density_values, pattern_values, and Age.

        Parameters
        ----------
        structure_graph_id : dict of integers
            what to retrieve
        intensity_values : array of strings, optional
            intensity values, 'High','Low', 'Medium' (default)
        density_values : array of strings, optional
            density values, 'High', 'Low'
        pattern_values : array of strings, optional
            pattern values, 'Full'
        age_names : array of strings, options
            for example 'E11.5', '13.5'

        Returns
        -------
        data : dict
            The parsed JSON response message.

        Notes
        -----
        This method uses the RMA endpoint to search annotated SectionDataSet data.
        '''
        age_include_strings = ['age']

        if age_names is not None and len(age_names) > 0:
            age_include_strings.append('[name$in')
            age_include_strings.append(
                ','.join(("'%s'" % (a) for a in age_names)))
            age_include_strings.append(']')
        age_include = ''.join(age_include_strings)

        criteria_strings = ['manual_annotations']

        if intensity_values is not None and len(intensity_values) > 0:
            criteria_strings.append('[intensity_call$in%s]' %
                                    (','.join(("'%s'" % (v) for v in intensity_values))))

        if density_values is not None and len(density_values) > 0:
            criteria_strings.append('[density_call$in%s]' %
                                    (','.join(("'%s'" % (v) for v in density_values))))

        if pattern_values is not None and len(pattern_values) > 0:
            criteria_strings.append('[pattern_call$in%s]' %
                                    (','.join(("'%s'" % (v) for v in pattern_values))))

        criteria_strings.append('(structure[id$in%s])' %
                                (','.join((str(s) for s in structures))))

        criteria_clause = ''.join(criteria_strings)

        include_clause = ''.join(['specimen',
                                  '(donor(',
                                  age_include,
                                  ')),',
                                  'probes(gene),'
                                  'plane_of_section'])

        order_by_array = ['genes.acronym',
                          'ages.embryonic+desc',
                          'ages.days',
                          'data_sets.id']

        data = self.model_query('SectionDataSet',
                                criteria=criteria_clause,
                                include=include_clause,
                                start_row=0,
                                num_rows=50,
                                order=order_by_array)

        return data

    def get_compound_annotated_section_data_sets(self,
                                                 queries,
                                                 fmt='json'):
        '''Find the SectionDataSet that matches several annotated_section_data_sets queries
        linked together with a Boolean 'and' or 'or'.

        Parameters
        ----------
        queries : array of dicts
            dicts with args like build_query
        fmt : string, optional
            'json' or 'xml'

        Returns
        -------
        data : dict
            The parsed JSON repsonse message.
        '''
        url_strings = ['?query=']

        for query in queries:
            url_strings.append('[')

            params = ['structures $in ' +
                      ','.join((str(s) for s in query['structures']))]

            for key in ['intensity_values', 'density_values', 'pattern_values', 'age_names']:
                if key in query and len(query[key]) > 0:
                    params.append('%s $in %s' %
                                  (key,
                                   ','.join(("'%s'" % (v) for v in query['intensity_values']))))

            url_strings.append(' : '.join(params))

            url_strings.append(']')

            if 'link' in query and query['link'] == 'or':
                url_strings.append(' or ')
            if 'link' in query and query['link'] == 'and':
                url_strings.append(' and ')

        url_params = ''.join(url_strings)

        url = ''.join([self.compound_annotated_section_data_sets_endpoint,
                       '.',
                       fmt,
                       url_params])

        return self.json_msg_query(url)



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import os

import pandas as pd
import pandas.io.json as pj

import pytest
from mock import MagicMock

from allensdk.api.cache import Cache, memoize
from allensdk.api.queries.rma_api import RmaApi
import allensdk.core.json_utilities as ju
from allensdk.config.manifest import ManifestVersionError
from allensdk.config.manifest_builder import ManifestBuilder

from allensdk.test_utilities.temp_dir import fn_temp_dir


@pytest.fixture
def cache():
    return Cache()


@pytest.fixture
def rma():
    return RmaApi()


def test_version_update(fn_temp_dir):

    class DummyCache(Cache):
        
        VERSION = None
    
        def build_manifest(self, file_name):
            manifest_builder = ManifestBuilder()
            manifest_builder.set_version(DummyCache.VERSION)
            manifest_builder.write_json_file(file_name)

    mpath = os.path.join(fn_temp_dir, 'manifest.json')
    dc = DummyCache(manifest=mpath)
    
    same_dc = DummyCache(manifest=mpath)
    
    with pytest.raises(ManifestVersionError):
        new_dc = DummyCache(manifest=mpath, version=1.0)


def test_wrap_json(rma, cache):
    msg = [{'whatever': True}]

    ju.read_url_get = \
        MagicMock(name='read_url_get',
                  return_value={'msg': msg})
    ju.write = \
        MagicMock(name='write')

    ju.read = \
        MagicMock(name='read',
                  return_value=pd.DataFrame(msg))

    df = cache.wrap(rma.model_query,
                    'example.txt',
                    cache=True,
                    model='Hemisphere')

    assert df.loc[:, 'whatever'][0]

    ju.read_url_get.assert_called_once_with(
        'http://api.brain-map.org/api/v2/data/query.json?q=model::Hemisphere')
    ju.write.assert_called_once_with('example.txt', msg)
    ju.read.assert_called_once_with('example.txt')


def test_wrap_dataframe(rma, cache):
    msg = [{'whatever': True}]

    ju.read_url_get = \
        MagicMock(name='read_url_get',
                  return_value={'msg': msg})
    ju.write = \
        MagicMock(name='write')
    pj.read_json = \
        MagicMock(name='read_json',
                  return_value=msg)

    json_data = cache.wrap(rma.model_query,
                           'example.txt',
                           cache=True,
                           return_dataframe=True,
                           model='Hemisphere')

    assert json_data[0]['whatever']

    ju.read_url_get.assert_called_once_with(
        'http://api.brain-map.org/api/v2/data/query.json?q=model::Hemisphere')
    ju.write.assert_called_once_with('example.txt', msg)
    pj.read_json.assert_called_once_with('example.txt', orient='records')

def test_memoize():

        import time

        @memoize
        def f(x):
            time.sleep(1)
            return x

        for ii in range(2):
            t0 = time.time()
            print f(0), time.time() - t0

        class FooBar(object):

            def __init__(self): pass

            @memoize
            def f(self, x):
                time.sleep(.1)
                return 1

        fb = FooBar()

        for ii in range(2):
            t0 = time.time()
            fb.f(0), time.time() - t0


# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import pytest
import mock
from numpy import allclose

from allensdk.api.queries.ontologies_api import OntologiesApi
from allensdk.core.structure_tree import StructureTree


@pytest.fixture
def nodes():

    return [{'id': 0, 'structure_id_path': [0], 'rgb_triplet': [0, 0, 0], 'acronym': 'rt', 'name': 'root', 'structure_set_ids':[1, 4]}, 
            {'id': 1, 'structure_id_path': [0, 1], 'rgb_triplet': [0, 15, 255], 'acronym': 'a', 'name': 'alpha', 'structure_set_ids': [1, 3]}, 
            {'id': 2, 'structure_id_path': [0, 2], 'rgb_triplet': [255, 255, 255], 'acronym': 'b', 'name': 'beta', 'structure_set_ids': [1, 2]}]


@pytest.fixture
def tree(nodes):
    return StructureTree(nodes)
    

@pytest.fixture
def oapi():
    oa = OntologiesApi()
    
    oa.get_structures = mock.MagicMock(return_value=[{'id': 1, 'structure_id_path': '1'}])
    oa.get_structure_set_map = mock.MagicMock(return_value={1: [2, 3]})
    
    return oa
    
    
def test_get_structures_by_id(tree):
    
    obtained = tree.get_structures_by_id([1, 2])
    assert( len(obtained) == 2 ) 
    
    
def test_get_structures_by_name(tree):
    
    obtained = tree.get_structures_by_name(['root'])
    assert( len(obtained) == 1 )
    
    
def test_get_structures_by_acronym(tree):

    obtained = tree.get_structures_by_acronym(['rt', 'a', 'b'])
    assert( len(obtained) == 3)
    

def test_get_structures_by_set_id(tree):

    obtained = tree.get_structures_by_set_id([2, 3])

    assert( len(obtained) == 2 )
    
    
def test_get_colormap(tree):
    
    obtained = tree.get_colormap()
    assert( allclose(obtained[0], [0, 0, 0]) )
    assert( allclose(obtained[2], [255, 255, 255]) )   
    
    
def test_get_name_map(tree):
    
    obtained = tree.get_name_map()
    assert( obtained[0] == 'root' )
    assert( obtained[2] == 'beta' )  
    
    
def test_get_id_acronym_map(tree):
    
    obtained = tree.get_id_acronym_map()
    assert( obtained['rt'] == 0 )
    

def test_get_ancestor_id_map(tree):

    obtained = tree.get_ancestor_id_map()
    assert( set(obtained[2]) == set([2, 0]) )
    

def test_structure_descends_from(tree):
    
    assert( tree.structure_descends_from(2, 0) )
    assert( not tree.structure_descends_from(0, 1) )
    
    
def test_has_overlaps(tree):
    
    obtained = tree.has_overlaps([0, 1, 2])
    assert( obtained == set([0]) )
    
    obag = tree.has_overlaps([1, 2])
    assert( not obag )


def test_clean_structures(nodes):

    dirty_node = {'id': 0, 'structure_id_path': '/0/', 
                  'color_hex_triplet': '000000', 'acronym': 'rt', 
                  'name': 'root', 'structure_sets':[{'id': 1}, {'id': 4}]}
                  
    clean_node = StructureTree.clean_structures([dirty_node])[0]
    assert( isinstance(clean_node['rgb_triplet'], list) )
    assert( isinstance(clean_node['structure_id_path'], list) )
    
    
def test_clean_structures_no_sets():
    
    dirty_node = {'id': 0, 'structure_id_path': '/0/', 
                  'color_hex_triplet': '000000', 'acronym': 'rt', 
                  'name': 'root'}
    
    clean_node = StructureTree.clean_structures([dirty_node])
    st = StructureTree(clean_node)
    
    assert( len(clean_node[0]['structure_set_ids']) == 0 )
    
    
def test_clean_structures_only_ids():
    
    dirty_node = {'id': 0, 'structure_id_path': '/0/', 
                  'color_hex_triplet': '000000', 'acronym': 'rt', 
                  'name': 'root', 'structure_set_ids': [1, 2, 3] }
    
    clean_node = StructureTree.clean_structures([dirty_node])
    st = StructureTree(clean_node)
    
    assert( len(clean_node[0]['structure_set_ids']) == 3 )
    
    
def test_clean_structures_ids_sets():
    
    dirty_node = {'id': 0, 'structure_id_path': '/0/', 
                  'color_hex_triplet': '000000', 'acronym': 'rt', 
                  'name': 'root', 'structure_set_ids': [1, 2, 3], 
                  'structure_sets': [{'id': 1}, {'id': 4}] }
    
    clean_node = StructureTree.clean_structures([dirty_node])
    st = StructureTree(clean_node)
    
    assert( len(clean_node[0]['structure_set_ids']) == 4 )
    
    
def test_clean_structures_str_id():

    dirty_node = {'id': '0', 'structure_id_path': '/0/', 
                  'color_hex_triplet': '000000', 'acronym': 'rt', 
                  'name': 'root', 'structure_set_ids': [1, 2, 3], 
                  'structure_sets': [{'id': 1}, {'id': 4}] }
    
    clean_node = StructureTree.clean_structures([dirty_node])
    st = StructureTree(clean_node)
    
    assert( set(st.node_ids()) == set([0]) )
    
    
def test_get_structure_sets(tree):

    expected = set([1, 2, 3, 4])
    obtained = tree.get_structure_sets()
    assert( expected == obtained )


def test_clean_structures_weird_keys():
    
    dirty_node = {'id': 5, 'dummy_key': 'dummy_val'}
    clean_node = StructureTree.clean_structures([dirty_node])[0]

    assert( len(clean_node) == 2 )
    assert( clean_node['id'] == 5 )


@pytest.mark.parametrize('inp,out', [('990099', [153, 0, 153]), 
                                     ('#990099', [153, 0, 153]), 
                                     ([153, 0, 153], [153, 0, 153]), 
                                     ((153., 0., 153.), [153, 0, 153]), 
                                     ([long(153), long(0), long(153)], [153, 0, 153])])
def test_hex_to_rgb(inp, out):
    obt = StructureTree.hex_to_rgb(inp)
    assert(allclose(obt, out))


@pytest.mark.parametrize('inp,out', [('/1/2/3/', [1, 2, 3]),
                                     ('1/2/3/', [1, 2, 3]), 
                                     ('/1/2/3', [1, 2, 3]), 
                                     ('1/2/3', [1, 2, 3]), 
                                     ([1, 2, 3], [1, 2, 3]),
                                     ([1.0, long(2), 3], [1, 2, 3]), 
                                     ((1, 2, 3), [1, 2, 3]), 
                                     ('', [])])
def test_path_to_list(inp, out):
    obt = StructureTree.path_to_list(inp)
    assert(allclose(obt, out))



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
import pandas as pd
from six import string_types
from .rma_template import RmaTemplate
from ..cache import cacheable, Cache
from .rma_pager import pageable
from allensdk.config.manifest import Manifest
import allensdk.brain_observatory.stimulus_info as stimulus_info
import logging


class BrainObservatoryApi(RmaTemplate):
    _log = logging.getLogger('allensdk.api.queries.brain_observatory_api')

    NWB_FILE_TYPE = 'NWBOphys'
    CELL_MAPPING_ID = 590985414

    rma_templates = \
        {"brain_observatory_queries": [
            {'name': 'list_isi_experiments',
             'description': 'see name',
             'model': 'IsiExperiment',
             'num_rows': 'all',
             'count': False,
             'criteria_params': []
             },
            {'name': 'isi_experiment_by_ids',
             'description': 'see name',
             'model': 'IsiExperiment',
             'criteria': '[id$in{{ isi_experiment_ids }}]',
             'include': 'experiment_container(ophys_experiments,targeted_structure)',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['isi_experiment_ids']
             },
            {'name': 'ophys_experiment_by_ids',
             'description': 'see name',
             'model': 'OphysExperiment',
             'criteria': '{% if ophys_experiment_ids is defined %}[id$in{{ ophys_experiment_ids }}]{%endif%}',
             'include': 'experiment_container,well_known_files(well_known_file_type),targeted_structure,specimen(donor(age,transgenic_lines))',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['ophys_experiment_ids']
             },
            {'name': 'ophys_experiment_data',
             'description': 'see name',
             'model': 'WellKnownFile',
             'criteria': '[attachable_id$eq{{ ophys_experiment_id }}],well_known_file_type[name$eq%s]' % NWB_FILE_TYPE,
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['ophys_experiment_id']
             },
            {'name': 'column_definitions',
             'description': 'see name',
             'model': 'ApiColumnDefinition',
             'criteria': '[api_class_name$eq{{ api_class_name }}]',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['api_class_name']
             },
            {'name': 'column_definition_class_names',
             'description': 'see name',
             'model': 'ApiColumnDefinition',
             'only': ['api_class_name'],
             'num_rows': 'all',
             'count': False,
             },
            {'name': 'stimulus_mapping',
             'description': 'see name',
             'model': 'ApiCamStimulusMapping',
             'criteria': '{% if stimulus_mapping_ids is defined %}[id$in{{ stimulus_mapping_ids }}]{%endif%}',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['stimulus_mapping_ids']
             },
            {'name': 'experiment_container',
             'description': 'see name',
             'model': 'ExperimentContainer',
             'criteria': '{% if experiment_container_ids is defined %}[id$in{{ experiment_container_ids }}]{%endif%}',
             'include': 'ophys_experiments,isi_experiment,specimen(donor(conditions,age,transgenic_lines)),targeted_structure',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['experiment_container_ids']
             },
            {'name': 'experiment_container_metric',
             'description': 'see name',
             'model': 'ApiCamExperimentContainerMetric',
             'criteria': '{% if experiment_container_metric_ids is defined %}[id$in{{ experiment_container_metric_ids }}]{%endif%}',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['experiment_container_metric_ids']
             },
            {'name': 'cell_metric',
             'description': 'see name',
             'model': 'ApiCamCellMetric',
             'criteria': '{% if cell_specimen_ids is defined %}[cell_specimen_id$in{{ cell_specimen_ids }}]{%endif%}',
             'criteria_params': ['cell_specimen_ids']
             },
            {'name': 'cell_specimen_id_mapping_table',
             'description': 'see name',
             'model': 'WellKnownFile',
             'criteria': '[id$eq{{ mapping_table_id }}],well_known_file_type[name$eqOphysCellSpecimenIdMapping]',
             'num_rows': 'all',
             'count': False,
             'criteria_params': ['mapping_table_id']}
        ]}

    _QUERY_TEMPLATES = {
        "=": '({0} == {1})',
        "<": '({0} < {1})',
        ">": '({0} > {1})',
        "<=": '({0} <= {1})',
        ">=": '({0} >= {1})',
        "between": '({0} >= {1}) and ({0} <= {2})',
        "in": '({0} == {1})',
        "is": '({0} == {1})'
    }

    def __init__(self, base_uri=None, datacube_uri=None):
        super(BrainObservatoryApi, self).__init__(base_uri,
                                                  query_manifest=BrainObservatoryApi.rma_templates)

        self.datacube_uri = datacube_uri

    @cacheable()
    def get_ophys_experiments(self, ophys_experiment_ids=None):
        ''' Get OPhys Experiments by id

        Parameters
        ----------
        ophys_experiment_ids : integer or list of integers, optional
            only select specific experiments.

        Returns
        -------
        dict : ophys experiment metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'ophys_experiment_by_ids',
                                   ophys_experiment_ids=ophys_experiment_ids)

        return data

    def get_isi_experiments(self, isi_experiment_ids=None):
        ''' Get ISI Experiments by id

        Parameters
        ----------
        isi_experiment_ids : integer or list of integers, optional
            only select specific experiments.

        Returns
        -------
        dict : isi experiment metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'isi_experiment_by_ids',
                                   isi_experiment_ids=isi_experiment_ids)

        return data

    def list_isi_experiments(self, isi_ids=None):
        '''List ISI experiments available through the Allen Institute API

        Parameters
        ----------
        neuronal_model_ids : integer or list of integers, optional
            only select specific isi experiments.

        Returns
        -------
        dict : neuronal model metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'list_isi_experiments')

        return data

    def list_column_definition_class_names(self):
        ''' Get column definitions

        Parameters
        ----------

        Returns
        -------
        list : api class name strings
        '''
        data = self.template_query('brain_observatory_queries',
                                   'column_definition_class_names')

        names = list(set([n['api_class_name'] for n in data]))

        return names

    def get_column_definitions(self, api_class_name=None):
        ''' Get column definitions

        Parameters
        ----------
        api_class_names : string or list of strings, optional
            only select specific column definition records.

        Returns
        -------
        dict : column definition metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'column_definitions',
                                   api_class_name=api_class_name)

        return data

    @cacheable()
    def get_stimulus_mappings(self, stimulus_mapping_ids=None):
        ''' Get stimulus mappings by id

        Parameters
        ----------
        stimulus_mapping_ids : integer or list of integers, optional
            only select specific stimulus mapping records.

        Returns
        -------
        dict : stimulus mapping metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'stimulus_mapping',
                                   stimulus_mapping_ids=stimulus_mapping_ids)

        return data

    @cacheable()
    @pageable(num_rows=2000, total_rows='all')
    def get_cell_metrics(self, cell_specimen_ids=None, *args, **kwargs):
        ''' Get cell metrics by id

        Parameters
        ----------
        cell_metrics_ids : integer or list of integers, optional
            only select specific cell metric records.

        Returns
        -------
        dict : cell metric metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'cell_metric',
                                   cell_specimen_ids=cell_specimen_ids,
                                   *args,
                                   **kwargs)

        return data

    @cacheable()
    def get_experiment_containers(self, experiment_container_ids=None):
        ''' Get experiment container by id

        Parameters
        ----------
        experiment_container_ids : integer or list of integers, optional
            only select specific experiment containers.

        Returns
        -------
        dict : experiment container metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'experiment_container',
                                   experiment_container_ids=experiment_container_ids)

        return data

    def get_experiment_container_metrics(self, experiment_container_metric_ids=None):
        ''' Get experiment container metrics by id

        Parameters
        ----------
        isi_experiment_ids : integer or list of integers, optional
            only select specific experiments.

        Returns
        -------
        dict : isi experiment metadata
        '''
        data = self.template_query('brain_observatory_queries',
                                   'experiment_container_metric',
                                   experiment_container_metric_ids=experiment_container_metric_ids)

        return data

    @cacheable(strategy='create',
               pathfinder=Cache.pathfinder(file_name_position=2,
                                           path_keyword='file_name'))
    def save_ophys_experiment_data(self, ophys_experiment_id, file_name):
        data = self.template_query('brain_observatory_queries',
                                   'ophys_experiment_data',
                                   ophys_experiment_id=ophys_experiment_id)

        try:
            file_url = data[0]['download_link']
        except Exception as _:
            raise Exception("ophys experiment %d has no data file" %
                            ophys_experiment_id)

        self._log.warning(
            "Downloading ophys_experiment %d NWB. This can take some time." % ophys_experiment_id)

        self.retrieve_file_over_http(self.api_url + file_url, file_name)

    def filter_experiment_containers(self, containers,
                                     ids=None,
                                     targeted_structures=None,
                                     imaging_depths=None,
                                     transgenic_lines=None,
                                     include_failed=False):

        if not include_failed:
            containers = [c for c in containers if not c.get('failed', False)]

        if ids is not None:
            containers = [c for c in containers if c['id'] in ids]

        if targeted_structures is not None:
            containers = [c for c in containers if c[
                'targeted_structure']['acronym'] in targeted_structures]

        if imaging_depths is not None:
            containers = [c for c in containers if c[
                'imaging_depth'] in imaging_depths]

        if transgenic_lines is not None:
            containers = [c for c in containers for tl in c['specimen'][
                'donor']['transgenic_lines'] if tl['name'] in transgenic_lines]

        return containers

    def filter_ophys_experiments(self, experiments,
                                 ids=None,
                                 experiment_container_ids=None,
                                 targeted_structures=None,
                                 imaging_depths=None,
                                 transgenic_lines=None,
                                 stimuli=None,
                                 session_types=None,
                                 include_failed=False):

        # re-using the code from above
        experiments = self.filter_experiment_containers(experiments,
                                                        ids=ids,
                                                        targeted_structures=targeted_structures,
                                                        imaging_depths=imaging_depths,
                                                        transgenic_lines=transgenic_lines)

        if not include_failed:
            experiments = [e for e in experiments 
                           if not e.get('experiment_container',{}).get('failed', False)]

        if experiment_container_ids is not None:
            experiments = [e for e in experiments if e[
                'experiment_container_id'] in experiment_container_ids]

        if session_types is not None:
            experiments = [e for e in experiments if e[
                'stimulus_name'] in session_types]

        if stimuli is not None:
            experiments = [e for e in experiments
                           if len(set(stimuli) & set(stimulus_info.stimuli_in_session(e['stimulus_name']))) > 0]

        return experiments

    def filter_cell_specimens(self, cell_specimens,
                              ids=None,
                              experiment_container_ids=None,
                              include_failed=False,
                              filters=None):
        """
        Filter a list of cell specimen records returned from the get_cell_metrics method according 
        some of their properties.

        Parameters
        ----------
        cell_specimens: list of dicts
            List of records returned by the get_cell_metrics method.

        ids: list of integers
            Return only records for cells with cell specimen ids in this list

        experiment_container_ids: list of integers
            Return only records for cells that belong to experiment container ids in this list

        include_failed: bool
            Whether to include cells from failed experiment containers

        filters: list of dicts
            Custom query used to reproduce filter sets created in the Allen Brain Observatory
            web application.  The general form is a list of dictionaries each of which
            describes a filtering operation based on a metric.  For more information, see
            dataframe_query.  
        """

        if not include_failed:
            cell_specimens = [c for c in cell_specimens if not c.get(
                    'failed_experiment_container', False)]

        if ids is not None:
            cell_specimens = [c for c in cell_specimens if c[
                'cell_specimen_id'] in ids]

        if experiment_container_ids is not None:
            cell_specimens = [c for c in cell_specimens if c[
                'experiment_container_id'] in experiment_container_ids]

        if filters is not None:
            cell_specimens = self.dataframe_query(cell_specimens,
                                                  filters,
                                                  'cell_specimen_id')

        return cell_specimens

    def dataframe_query_string(self,
                               filters):
        """
        Convert a list of cell metric filter dictionaries into a 
        Pandas query string.
        """

        def _quote_string(v):
            if isinstance(v, string_types):
                return "'%s'" % (v)
            else:
                return str(v)

        def _filter_clause(op, field, value):
            if op == 'in':
                query_args = [field, str(value)]
            elif type(value) is list:
                query_args = [field] + list(map(_quote_string, value))
            else:
                query_args = [field, str(value)]

            cluster_string = self._QUERY_TEMPLATES[op].\
                format(*query_args)

            return cluster_string

        query_string = ' & '.join(_filter_clause(f['op'],
                                                 f['field'],
                                                 f['value']) for f in filters)

        return query_string

    def dataframe_query(self,
                        data,
                        filters,
                        primary_key):
        """
        Given a list of dictionary records and a list of filter dictionaries,
        filter the records using Pandas and return the filtered set of records.
        
        Parameters
        ----------
        data: list of dicts
           List of dictionaries

        filters: list of dicts
           Each dictionary describes a filtering operation on a field in the dictionary.
           The general form is { 'field': <field>, 'op': <operation>, 'value': <filter_value(s)> }.
           For example, you can apply a threshold on the "osi_dg" column with something like this:
           { 'field': 'osi_dg', 'op': '>', 'value': 1.0 }.  See _QUERY_TEMPLATES for a full list
           of operators.
        """
        queries = self.dataframe_query_string(filters)
        result_dataframe = pd.DataFrame(data)
        result_dataframe = result_dataframe.query(queries)

        result_keys = set(result_dataframe[primary_key])
        result = [d for d in data
                  if d[primary_key]
                  in result_keys]

        return result

    def get_cell_specimen_id_mapping(self, file_name, mapping_table_id=None):
        '''Download mapping table from old to new cell specimen IDs.

        The mapping table is a CSV file that maps cell specimen ids
        that have changed between processing runs of the Brain
        Observatory pipeline.

        Parameters
        ----------
        file_name : string
            Filename to save locally.
        mapping_table_id : integer
            ID of the mapping table file. Defaults to the most recent
            mapping table. 

        Returns
        -------
        pandas.DataFrame
            Mapping table as a DataFrame.
        '''
        if mapping_table_id is None:
            mapping_table_id = self.CELL_MAPPING_ID
        data = self.template_query('brain_observatory_queries',
                                   'cell_specimen_id_mapping_table',
                                   mapping_table_id=mapping_table_id)

        try:
            file_url = data[0]['download_link']
        except Exception as _:
            raise Exception("No OphysCellSpecimenIdMapping file found.")

        self.retrieve_file_over_http(self.api_url + file_url, file_name)
        return pd.read_csv(file_name)



# Allen Institute Software License - This software license is the 2-clause BSD
# license plus a third clause that prohibits redistribution for commercial
# purposes without further permission.
#
# Copyright 2016-2017. Allen Institute. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# 3. Redistributions for commercial purposes are not permitted without the
# Allen Institute's written permission.
# For purposes of this license, commercial purposes is the incorporation of the
# Allen Institute's software into anything for which you will charge fees or
# other compensation. Contact terms@alleninstitute.org for commercial licensing
# opportunities.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
from .stimulus_analysis import StimulusAnalysis
import scipy.stats as st
import pandas as pd
import numpy as np
import h5py
from math import sqrt
import logging
from . import observatory_plots as oplots
from . import circle_plots as cplots
from .brain_observatory_exceptions import MissingStimulusException
import matplotlib.pyplot as plt

class DriftingGratings(StimulusAnalysis):
    """ Perform tuning analysis specific to drifting gratings stimulus.

    Parameters
    ----------
    data_set: BrainObservatoryNwbDataSet object
    """

    _log = logging.getLogger('allensdk.brain_observatory.drifting_gratings')

    def __init__(self, data_set, **kwargs):
        super(DriftingGratings, self).__init__(data_set, **kwargs)

        self.sweeplength = 60
        self.interlength = 30
        self.extralength = 0

        self._orivals = DriftingGratings._PRELOAD
        self._tfvals = DriftingGratings._PRELOAD
        self._number_ori = DriftingGratings._PRELOAD
        self._number_tf = DriftingGratings._PRELOAD

    @property
    def orivals(self):
        if self._orivals is DriftingGratings._PRELOAD:
            self.populate_stimulus_table()

        return self._orivals

    @property
    def tfvals(self):
        if self._tfvals is DriftingGratings._PRELOAD:
            self.populate_stimulus_table()

        return self._tfvals

    @property
    def number_ori(self):
        if self._number_ori is DriftingGratings._PRELOAD:
            self.populate_stimulus_table()

        return self._number_ori

    @property
    def number_tf(self):
        if self._number_tf is DriftingGratings._PRELOAD:
            self.populate_stimulus_table()

        return self._number_tf

    def populate_stimulus_table(self):
        stimulus_table = self.data_set.get_stimulus_table('drifting_gratings')
        self._stim_table = stimulus_table.fillna(value=0.)
        self._orivals = np.unique(self.stim_table.orientation).astype(int)
        self._tfvals = np.unique(self.stim_table.temporal_frequency).astype(int)
        self._number_ori = len(self.orivals)
        self._number_tf = len(self.tfvals)

    def get_response(self):
        ''' Computes the mean response for each cell to each stimulus condition.  Return is
        a (# orientations, # temporal frequencies, # cells, 3) np.ndarray.  The final dimension
        contains the mean response to the condition (index 0), standard error of the mean of the response
        to the condition (index 1), and the number of trials with a significant response (p < 0.05)
        to that condition (index 2).

        Returns
        -------
        Numpy array storing mean responses.
        '''
        DriftingGratings._log.info("Calculating mean responses")

        response = np.empty(
            (self.number_ori, self.number_tf, self.numbercells + 1, 3))

        def ptest(x):
            return len(np.where(x < (0.05 / (8 * 5)))[0])

        for ori in self.orivals:
            ori_pt = np.where(self.orivals == ori)[0][0]
            for tf in self.tfvals:
                tf_pt = np.where(self.tfvals == tf)[0][0]
                subset_response = self.mean_sweep_response[
                    (self.stim_table.temporal_frequency == tf) & (self.stim_table.orientation == ori)]
                subset_pval = self.pval[(self.stim_table.temporal_frequency == tf) & (
                    self.stim_table.orientation == ori)]
                response[ori_pt, tf_pt, :, 0] = subset_response.mean(axis=0)
                response[ori_pt, tf_pt, :, 1] = subset_response.std(
                    axis=0) / sqrt(len(subset_response))
                response[ori_pt, tf_pt, :, 2] = subset_pval.apply(
                    ptest, axis=0)
        return response

    def get_peak(self):
        ''' Computes metrics related to each cell's peak response condition.

        Returns
        -------
        Pandas data frame containing the following columns (_dg suffix is
        for drifting grating):
            * ori_dg (orientation)
            * tf_dg (temporal frequency)
            * reliability_dg
            * osi_dg (orientation selectivity index)
            * dsi_dg (direction selectivity index)
            * peak_dff_dg (peak dF/F)
            * ptest_dg
            * p_run_dg
            * run_modulation_dg
            * cv_dg (circular variance)
        '''
        DriftingGratings._log.info('Calculating peak response properties')

        peak = pd.DataFrame(index=range(self.numbercells), columns=('ori_dg', 'tf_dg', 'reliability_dg',
                                                                    'osi_dg', 'dsi_dg', 'peak_dff_dg', 
                                                                    'ptest_dg', 'p_run_dg', 'run_modulation_dg', 
                                                                    'cv_os_dg', 'cv_ds_dg', 'tf_index_dg', 
                                                                    'cell_specimen_id'))
        cids = self.data_set.get_cell_specimen_ids()

        orivals_rad = np.deg2rad(self.orivals)        
        for nc in range(self.numbercells):
            cell_peak = np.where(self.response[:, 1:, nc, 0] == np.nanmax(
                self.response[:, 1:, nc, 0]))
            prefori = cell_peak[0][0]
            preftf = cell_peak[1][0] + 1
            peak.cell_specimen_id.iloc[nc] = cids[nc]
            peak.ori_dg.iloc[nc] = prefori
            peak.tf_dg.iloc[nc] = preftf

            pref = self.response[prefori, preftf, nc, 0]
            orth1 = self.response[np.mod(prefori + 2, 8), preftf, nc, 0]
            orth2 = self.response[np.mod(prefori - 2, 8), preftf, nc, 0]
            orth = (orth1 + orth2) / 2
            null = self.response[np.mod(prefori + 4, 8), preftf, nc, 0]

            tuning = self.response[:, preftf, nc, 0]
            tuning = np.where(tuning>0, tuning, 0)
            #new circular variance below
            CV_top_os = np.empty((8), dtype=np.complex128)
            CV_top_ds = np.empty((8), dtype=np.complex128)
            for i in range(8):
                CV_top_os[i] = (tuning[i]*np.exp(1j*2*orivals_rad[i]))
                CV_top_ds[i] = (tuning[i]*np.exp(1j*orivals_rad[i]))
            peak.cv_os_dg.iloc[nc] = np.abs(CV_top_os.sum())/tuning.sum()
            peak.cv_ds_dg.iloc[nc] = np.abs(CV_top_ds.sum())/tuning.sum()
            
            peak.osi_dg.iloc[nc] = (pref - orth) / (pref + orth)
            peak.dsi_dg.iloc[nc] = (pref - null) / (pref + null)
            peak.peak_dff_dg.iloc[nc] = pref

            groups = []
            for ori in self.orivals:
                for tf in self.tfvals[1:]:
                    groups.append(self.mean_sweep_response[(self.stim_table.temporal_frequency == tf) & (
                        self.stim_table.orientation == ori)][str(nc)])
            groups.append(self.mean_sweep_response[
                          self.stim_table.temporal_frequency == 0][str(nc)])
            _, p = st.f_oneway(*groups)
            peak.ptest_dg.iloc[nc] = p

            subset = self.mean_sweep_response[(self.stim_table.temporal_frequency == self.tfvals[
                                               preftf]) & (self.stim_table.orientation == self.orivals[prefori])]
            #running modulation                                 
            subset_stat = subset[subset.dx < 1]
            subset_run = subset[subset.dx >= 1]
            if (len(subset_run) > 2) & (len(subset_stat) > 2):
                (_,peak.p_run_dg.iloc[nc]) = st.ttest_ind(subset_run[str(nc)], subset_stat[str(nc)], equal_var=False)
                
                if subset_run[str(nc)].mean()>subset_stat[str(nc)].mean():
                    peak.run_modulation_dg.iloc[nc] = (subset_run[str(nc)].mean() - subset_stat[str(nc)].mean())/np.abs(subset_run[str(nc)].mean())
                elif subset_run[str(nc)].mean()<subset_stat[str(nc)].mean():
                    peak.run_modulation_dg.iloc[nc] = -1*((subset_stat[str(nc)].mean() - subset_run[str(nc)].mean())/np.abs(subset_stat[str(nc)].mean()))

            else:
                peak.p_run_dg.iloc[nc] = np.NaN
                peak.run_modulation_dg.iloc[nc] = np.NaN
            
            #reliability 
            subset = self.sweep_response[(self.stim_table.temporal_frequency == self.tfvals[
                                               preftf]) & (self.stim_table.orientation == self.orivals[prefori])]
            corr_matrix = np.empty((len(subset),len(subset)))
            for i in range(len(subset)):
                for j in range(len(subset)):
                    r,p = st.pearsonr(subset[str(nc)].iloc[i][30:90], subset[str(nc)].iloc[j][30:90])
                    corr_matrix[i,j] = r
            mask = np.ones((len(subset), len(subset)))
            for i in range(len(subset)):
                for j in range(len(subset)):
                    if i>=j:
                        mask[i,j] = np.NaN
            corr_matrix *= mask
            peak.reliability_dg.iloc[nc] = np.nanmean(corr_matrix)
            
            #TF index
            tf_tuning = self.response[prefori,1:,nc,0]
            trials = self.mean_sweep_response[(self.stim_table.temporal_frequency!=0)&(self.stim_table.orientation==self.orivals[prefori])][str(nc)].values
            SSE_part = np.sqrt(np.sum((trials-trials.mean())**2)/(len(trials)-5))
            peak.tf_index_dg.iloc[nc] = (np.ptp(tf_tuning))/(np.ptp(tf_tuning) + 2*SSE_part)

        return peak

    def open_star_plot(self, cell_specimen_id=None, include_labels=False, cell_index=None):
        cell_index = self.row_from_cell_id(cell_specimen_id, cell_index)

        df = self.mean_sweep_response[str(cell_index)]
        st = self.data_set.get_stimulus_table('drifting_gratings')
        mask = st.dropna(subset=['orientation']).index
        
        data = df.values
    
        cmin = self.response[0,0,cell_index,0]
        cmax = data.mean() + data.std()*3

        fp = cplots.FanPlotter.for_drifting_gratings()
        fp.plot(r_data=st.temporal_frequency.ix[mask].values,
                angle_data=st.orientation.ix[mask].values,
                data=df.ix[mask].values,
                clim=[cmin, cmax])
        fp.show_axes(closed=True)
    
        if include_labels:
            fp.show_r_labels()
            fp.show_angle_labels()

    def plot_orientation_selectivity(self,
                                     si_range=oplots.SI_RANGE,
                                     n_hist_bins=oplots.N_HIST_BINS,
                                     color=oplots.STIM_COLOR,
                                     p_value_max=oplots.P_VALUE_MAX,
                                     peak_dff_min=oplots.PEAK_DFF_MIN):
        # responsive cells 
        vis_cells = (self.peak.ptest_dg < p_value_max) & (self.peak.peak_dff_dg > peak_dff_min)

        # orientation selective cells
        osi_cells = vis_cells & (self.peak.osi_dg > si_range[0]) & (self.peak.osi_dg < si_range[1])

        peak_osi = self.peak.ix[osi_cells]
        osis = peak_osi.osi_dg.values

        oplots.plot_selectivity_cumulative_histogram(osis, 
                                                     "orientation selectivity index",
                                                     si_range=si_range,
                                                     n_hist_bins=n_hist_bins,
                                                     color=color)

    def plot_direction_selectivity(self,
                                   si_range=oplots.SI_RANGE,
                                   n_hist_bins=oplots.N_HIST_BINS,
                                   color=oplots.STIM_COLOR,
                                   p_value_max=oplots.P_VALUE_MAX,
                                   peak_dff_min=oplots.PEAK_DFF_MIN):

        # responsive cells 
        vis_cells = (self.peak.ptest_dg < p_value_max) & (self.peak.peak_dff_dg > peak_dff_min)

        # direction selective cells
        dsi_cells = vis_cells & (self.peak.dsi_dg > si_range[0]) & (self.peak.dsi_dg < si_range[1])

        peak_dsi = self.peak.ix[dsi_cells]
        dsis = peak_dsi.dsi_dg.values

        oplots.plot_selectivity_cumulative_histogram(dsis, 
                                                     "direction selectivity index",
                                                     si_range=si_range,
                                                     n_hist_bins=n_hist_bins,
                                                     color=color)

    def plot_preferred_direction(self,
                                 include_labels=False,
                                 si_range=oplots.SI_RANGE,
                                 color=oplots.STIM_COLOR,
                                 p_value_max=oplots.P_VALUE_MAX,
                                 peak_dff_min=oplots.PEAK_DFF_MIN):
        vis_cells = (self.peak.ptest_dg < p_value_max) & (self.peak.peak_dff_dg > peak_dff_min)    
        pref_dirs = self.peak.ix[vis_cells].ori_dg.values
        pref_dirs = [ self.orivals[pref_dir] for pref_dir in pref_dirs ]

        angles, counts = np.unique(pref_dirs, return_counts=True)
        oplots.plot_radial_histogram(angles, 
                                     counts,
                                     include_labels=include_labels,
                                     all_angles=self.orivals,
                                     direction=-1,
                                     offset=0.0,
                                     closed=True,
                                     color=color)

    def plot_preferred_temporal_frequency(self, 
                                          si_range=oplots.SI_RANGE,
                                          color=oplots.STIM_COLOR,
                                          p_value_max=oplots.P_VALUE_MAX,
                                          peak_dff_min=oplots.PEAK_DFF_MIN):

        vis_cells = (self.peak.ptest_dg < p_value_max) & (self.peak.peak_dff_dg > peak_dff_min)    
        pref_tfs = self.peak.ix[vis_cells].tf_dg.values

        oplots.plot_condition_histogram(pref_tfs, 
                                        self.tfvals[1:],
                                        color=color)

        plt.xlabel("temporal frequency (Hz)")
        plt.ylabel("number of cells")

    def reshape_response_array(self):
        '''
        :return: response array in cells x stim x repetition for noise correlations
        '''

        mean_sweep_response = self.mean_sweep_response.values[:, :self.numbercells]

        reps = []
        stim_table = self.stim_table

        tfvals = self.tfvals
        tfvals = tfvals[tfvals != 0] # blank sweep

        response_new = np.zeros((self.numbercells, self.number_ori, self.number_tf-1), dtype='object')

        for i, ori in enumerate(self.orivals):
            for j, tf in enumerate(tfvals):
                    ind = (stim_table.orientation.values == ori) * (stim_table.temporal_frequency.values == tf)
                    for c in range(self.numbercells):
                        response_new[c, i, j] = mean_sweep_response[ind, c]

        ind = (stim_table.temporal_frequency.values == 0)
        response_blank = mean_sweep_response[ind, :].T

        return response_new, response_blank

    def get_signal_correlation(self, corr='spearman'):
        logging.debug("Calculating signal correlation")

        response = self.response[:, 1:, :self.numbercells, 0] # orientation x freq x cell, no blank
        response = response.reshape(self.number_ori * (self.number_tf-1), self.numbercells).T
        N, Nstim = response.shape

        signal_corr = np.zeros((N, N))
        signal_p = np.empty((N, N))
        if corr == 'pearson':
            for i in range(N):
                for j in range(i, N): # matrix is symmetric
                    signal_corr[i, j], signal_p[i, j] = st.pearsonr(response[i], response[j])

        elif corr == 'spearman':
            for i in range(N):
                for j in range(i, N): # matrix is symmetric
                    signal_corr[i, j], signal_p[i, j] = st.spearmanr(response[i], response[j])

        else:
            raise Exception('correlation should be pearson or spearman')

        signal_corr = np.triu(signal_corr) + np.triu(signal_corr, 1).T  # fill in lower triangle
        signal_p = np.triu(signal_p) + np.triu(signal_p, 1).T  # fill in lower triangle

        return signal_corr, signal_p


    def get_representational_similarity(self, corr='spearman'):
        logging.debug("Calculating representational similarity")

        response = self.response[:, 1:, :self.numbercells, 0] # orientation x freq x phase x cell, no blank
        response = response.reshape(self.number_ori * (self.number_tf-1), self.numbercells)
        Nstim, N = response.shape

        rep_sim = np.zeros((Nstim, Nstim))
        rep_sim_p = np.empty((Nstim, Nstim))
        if corr == 'pearson':
            for i in range(Nstim):
                for j in range(i, Nstim): # matrix is symmetric
                    rep_sim[i, j], rep_sim_p[i, j] = st.pearsonr(response[i], response[j])

        elif corr == 'spearman':
            for i in range(Nstim):
                for j in range(i, Nstim): # matrix is symmetric
                    rep_sim[i, j], rep_sim_p[i, j] = st.spearmanr(response[i], response[j])

        else:
            raise Exception('correlation should be pearson or spearman')

        rep_sim = np.triu(rep_sim) + np.triu(rep_sim, 1).T # fill in lower triangle
        rep_sim_p = np.triu(rep_sim_p) + np.triu(rep_sim_p, 1).T  # fill in lower triangle

        return rep_sim, rep_sim_p


    def get_noise_correlation(self, corr='spearman'):
        logging.debug("Calculating noise correlations")

        response, response_blank = self.reshape_response_array()
        noise_corr = np.zeros((self.numbercells, self.numbercells, self.number_ori, self.number_tf-1))
        noise_corr_p = np.zeros((self.numbercells, self.numbercells, self.number_ori, self.number_tf-1))

        noise_corr_blank = np.zeros((self.numbercells, self.numbercells))
        noise_corr_blank_p = np.zeros((self.numbercells, self.numbercells))

        if corr == 'pearson':
            for k in range(self.number_ori):
                for l in range(self.number_tf-1):
                    for i in range(self.numbercells):
                        for j in range(i, self.numbercells):
                            noise_corr[i, j, k, l], noise_corr_p[i, j, k, l] = st.pearsonr(response[i, k, l], response[j, k, l])

                    noise_corr[:, :, k, l] = np.triu(noise_corr[:, :, k, l]) + np.triu(noise_corr[:, :, k, l], 1).T

            for i in range(self.numbercells):
                for j in range(i, self.numbercells):
                    noise_corr_blank[i, j], noise_corr_blank_p[i, j] = st.pearsonr(response_blank[i], response_blank[j])

        elif corr == 'spearman':
            for k in range(self.number_ori):
                for l in range(self.number_tf-1):
                    for i in range(self.numbercells):
                        for j in range(i, self.numbercells):
                            noise_corr[i, j, k, l], noise_corr_p[i, j, k, l] = st.spearmanr(response[i, k, l], response[j, k, l])

                    noise_corr[:, :, k, l] = np.triu(noise_corr[:, :, k, l]) + np.triu(noise_corr[:, :, k, l], 1).T

            for i in range(self.numbercells):
                for j in range(i, self.numbercells):
                    noise_corr_blank[i, j], noise_corr_blank_p[i, j] = st.spearmanr(response_blank[i], response_blank[j])

        else:
            raise Exception('correlation should be pearson or spearman')

        noise_corr_blank[:, :] = np.triu(noise_corr_blank[:, :]) + np.triu(noise_corr_blank[:, :], 1).T

        return noise_corr, noise_corr_p, noise_corr_blank, noise_corr_blank_p


    @staticmethod
    def from_analysis_file(data_set, analysis_file):
        dg = DriftingGratings(data_set)

        try:
            dg.populate_stimulus_table()

            dg._sweep_response = pd.read_hdf(analysis_file, "analysis/sweep_response_dg")
            dg._mean_sweep_response = pd.read_hdf(analysis_file, "analysis/mean_sweep_response_dg")
            dg._peak = pd.read_hdf(analysis_file, "analysis/peak")

            with h5py.File(analysis_file, "r") as f:
                dg._response = f["analysis/response_dg"].value
                dg._binned_dx_sp = f["analysis/binned_dx_sp"].value
                dg._binned_cells_sp = f["analysis/binned_cells_sp"].value
                dg._binned_dx_vis = f["analysis/binned_dx_vis"].value
                dg._binned_cells_vis = f["analysis/binned_cells_vis"].value
                if "analysis/noise_corr_dg" in f:
                    dg.noise_correlation = f["analysis/noise_corr_dg"].value
                if "analysis/signal_corr_dg" in f:
                    dg.signal_correlation = f["analysis/signal_corr_dg"].value
                if "analysis/rep_similarity_dg" in f:
                    dg.representational_similarity = f["analysis/rep_similarity_dg"].value

        except Exception as e:
            raise MissingStimulusException(e.args)

        return dg

